@@ -19,16 +19,18 @@
 #ifndef YASH_ULIMIT_H
 #define YASH_ULIMIT_H
 
-#include "../util.h"
+#include "../xgetopt.h"
 
 
 extern int ulimit_builtin(int argc, void **argv)
     __attribute__((nonnull));
+#if YASH_ENABLE_HELP
 extern const char ulimit_help[], ulimit_syntax[];
+#endif
 extern const struct xgetopt_T ulimit_options[];
 
 
 #endif /* YASH_ULIMIT_H */
 
 
-/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
+/* vim: set ts=8 sts=4 sw=4 et tw=80: */
