@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* input.c: functions for input of command line */
-/* (C) 2007-2019 magicant */
+/* (C) 2007-2025 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,7 +54,8 @@
 
 static bool is_seekable_file(int fd);
 static inputresult_T optimized_read_input(
-	struct xwcsbuf_T *buf, struct input_file_info_T *info, _Bool trap)
+        struct xwcsbuf_T *buf, struct input_file_info_T *info, bool trap,
+        wchar_t delimiter)
     __attribute__((nonnull));
 static wchar_t *expand_prompt_variable(wchar_t num, wchar_t suffix)
     __attribute__((malloc,warn_unused_result));
@@ -64,6 +65,8 @@ static wchar_t *expand_ps1_posix(wchar_t *s)
     __attribute__((nonnull,malloc,warn_unused_result));
 static inline wchar_t get_euid_marker(void)
     __attribute__((pure));
+static wchar_t *post_prompt_command(wchar_t *line)
+    __attribute__((nonnull,malloc,warn_unused_result));
 
 /* An input function that inputs from a wide string.
  * `inputinfo' must be a pointer to a `struct input_wcs_info_T'.
@@ -75,14 +78,14 @@ inputresult_T input_wcs(struct xwcsbuf_T *buf, void *inputinfo)
     const wchar_t *src = info->src;
 
     if (src == NULL)
-	return INPUT_EOF;
+        return INPUT_EOF;
 
     const wchar_t *newlinep = wcschr(src, L'\n');
     if (newlinep != NULL) {
-	const wchar_t *nextlinep = &newlinep[1];
-	wb_ncat_force(buf, src, nextlinep - src);
-	info->src = nextlinep;
-	return INPUT_OK;
+        const wchar_t *nextlinep = &newlinep[1];
+        wb_ncat_force(buf, src, nextlinep - src);
+        info->src = nextlinep;
+        return INPUT_OK;
     }
 
     bool isempty = (src[0] == L'\0');
@@ -107,71 +110,79 @@ inputresult_T input_file(struct xwcsbuf_T *buf, void *inputinfo)
  *   INPUT_EOF   if reached the end of file without reading any characters
  *   INPUT_ERROR if an error occurred before reading any characters */
 inputresult_T read_input(
-	xwcsbuf_T *buf, struct input_file_info_T *info, bool trap)
+        xwcsbuf_T *buf, struct input_file_info_T *info, bool trap)
+{
+    return read_input_delimited(buf, info, trap, L'\n');
+}
+
+/* Like `read_input', but reads input until a delimiter character is found. */
+inputresult_T read_input_delimited(
+        xwcsbuf_T *buf, struct input_file_info_T *info, bool trap,
+        wchar_t delimiter)
 {
     if (info->bufsize == 1 && is_seekable_file(info->fd))
-	return optimized_read_input(buf, info, trap);
+        return optimized_read_input(buf, info, trap, delimiter);
 
     size_t initlen = buf->length;
     inputresult_T status = INPUT_EOF;
 
     for (;;) {
-	if (info->bufpos >= info->bufmax) {
+        if (info->bufpos >= info->bufmax) {
 read_input:  /* if there's nothing in the buffer, read the next input */
-	    switch (wait_for_input(info->fd, trap, -1)) {
-		case W_READY:
-		    break;
-		case W_TIMED_OUT:
-		    assert(false);
-		case W_INTERRUPTED:
-		    // Ignore interruption and continue reading, because:
-		    //  1) POSIX does not require to handle interruption, and
-		    //  2) the buffer for canonical-mode editing cannot be
-		    //     controlled from the shell.
-		    goto read_input;
-		case W_ERROR:
-		    status = INPUT_ERROR;
-		    goto end;
-	    }
+            switch (wait_for_input(info->fd, trap, -1)) {
+                case W_READY:
+                    break;
+                case W_TIMED_OUT:
+                    assert(false);
+                case W_INTERRUPTED:
+                    // Ignore interruption and continue reading, because:
+                    //  1) POSIX does not require to handle interruption, and
+                    //  2) the buffer for canonical-mode editing cannot be
+                    //     controlled from the shell.
+                    goto read_input;
+                case W_ERROR:
+                    status = INPUT_ERROR;
+                    goto end;
+            }
 
-	    ssize_t readcount = read(info->fd, info->buf, info->bufsize);
-	    if (readcount < 0) switch (errno) {
-		case EINTR:
-		case EAGAIN:
+            ssize_t readcount = read(info->fd, info->buf, info->bufsize);
+            if (readcount < 0) switch (errno) {
+                case EINTR:
+                case EAGAIN:
 #if EAGAIN != EWOULDBLOCK
-		case EWOULDBLOCK:
+                case EWOULDBLOCK:
 #endif
-		    goto read_input;  /* try again */
-		default:
-		    goto error;
-	    } else if (readcount == 0) {
-		goto end;
-	    }
-	    info->bufpos = 0;
-	    info->bufmax = readcount;
-	}
+                    goto read_input;  /* try again */
+                default:
+                    goto error;
+            } else if (readcount == 0) {
+                goto end;
+            }
+            info->bufpos = 0;
+            info->bufmax = readcount;
+        }
 
-	/* convert bytes in `info->buf' into a wide character and
-	 * append it to `buf' */
-	wb_ensuremax(buf, add(buf->length, 1));
-	assert(info->bufpos < info->bufmax);
-	size_t convcount = mbrtowc(&buf->contents[buf->length],
-		&info->buf[info->bufpos], info->bufmax - info->bufpos,
-		&info->state);
-	switch (convcount) {
-	    case 0:            /* read null character */
-		goto end;
-	    case (size_t) -1:  /* not a valid character */
-		goto error;
-	    case (size_t) -2:  /* needs more input */
-		goto read_input;
-	    default:
-		info->bufpos += convcount;
-		buf->contents[++buf->length] = L'\0';
-		if (buf->contents[buf->length - 1] == L'\n')
-		    goto end;
-		break;
-	}
+        /* convert bytes in `info->buf' into a wide character and
+         * append it to `buf' */
+        wb_ensuremax(buf, add(buf->length, 1));
+        assert(info->bufpos < info->bufmax);
+        size_t convcount = mbrtowc(&buf->contents[buf->length],
+                &info->buf[info->bufpos], info->bufmax - info->bufpos,
+                &info->state);
+        switch (convcount) {
+            case 0:            /* read null character */
+                goto end;
+            case (size_t) -1:  /* not a valid character */
+                goto error;
+            case (size_t) -2:  /* needs more input */
+                goto read_input;
+            default:
+                info->bufpos += convcount;
+                buf->contents[++buf->length] = L'\0';
+                if (buf->contents[buf->length - 1] == delimiter)
+                    goto end;
+                break;
+        }
     }
 
 error:
@@ -179,9 +190,9 @@ error:
     status = INPUT_ERROR;
 end:
     if (initlen != buf->length)
-	return INPUT_OK;
+        return INPUT_OK;
     else
-	return status;
+        return status;
 }
 
 /* Checks if the file descriptor is seekable. */
@@ -198,29 +209,30 @@ bool is_seekable_file(int fd)
  * once even if `info->bufsize' is 1. The input file descriptor must be
  * seekable. */
 inputresult_T optimized_read_input(
-	struct xwcsbuf_T *buf, struct input_file_info_T *info, _Bool trap)
+        struct xwcsbuf_T *buf, struct input_file_info_T *info, bool trap,
+        wchar_t delimiter)
 {
     struct input_file_info_T *tmpinfo =
-	xmallocs(sizeof *tmpinfo, BUFSIZ, sizeof *tmpinfo->buf);
+        xmallocs(sizeof *tmpinfo, BUFSIZ, sizeof *tmpinfo->buf);
     tmpinfo->fd = info->fd;
     tmpinfo->state = info->state;
     tmpinfo->bufpos = tmpinfo->bufmax = 0;
     tmpinfo->bufsize = BUFSIZ;
 
     while (info->bufpos < info->bufmax)
-	tmpinfo->buf[tmpinfo->bufmax++] = info->buf[info->bufpos++];
+        tmpinfo->buf[tmpinfo->bufmax++] = info->buf[info->bufpos++];
 
-    inputresult_T result = read_input(buf, tmpinfo, trap);
+    inputresult_T result = read_input_delimited(buf, tmpinfo, trap, delimiter);
 
     if (tmpinfo->bufpos < tmpinfo->bufmax) {
-	/* rewind the FD to pretend we're not buffering */
-	off_t diff = tmpinfo->bufmax - tmpinfo->bufpos;
-	if (lseek(tmpinfo->fd, -diff, SEEK_CUR) == (off_t) -1) {
-	    xerror(errno,
-		    Ngt("cannot rewind file descriptor %d after reading. "
-			"Subsequent reads may lack some text"),
-		    tmpinfo->fd);
-	}
+        /* rewind the FD to pretend we're not buffering */
+        off_t diff = tmpinfo->bufmax - tmpinfo->bufpos;
+        if (lseek(tmpinfo->fd, -diff, SEEK_CUR) == (off_t) -1) {
+            xerror(errno,
+                    Ngt("cannot rewind file descriptor %d after reading. "
+                        "Subsequent reads may lack some text"),
+                    tmpinfo->fd);
+        }
     }
 
     info->state = tmpinfo->state;
@@ -238,62 +250,66 @@ inputresult_T optimized_read_input(
 inputresult_T input_interactive(struct xwcsbuf_T *buf, void *inputinfo)
 {
     struct input_interactive_info_T *info = inputinfo;
-    struct promptset_T prompt;
-
     if (info->prompttype == 1) {
-	if (!posixly_correct)
-	    exec_variable_as_auxiliary_(VAR_PROMPT_COMMAND);
-	check_mail();
+        if (!posixly_correct)
+            exec_variable_as_auxiliary_(VAR_PROMPT_COMMAND);
+        check_mail();
     }
-    prompt = get_prompt(info->prompttype);
+
+    struct promptset_T prompt = get_prompt(info->prompttype);
     if (do_job_control)
-	print_job_status_all();
+        print_job_status_all();
     /* Note: no commands must be executed between `print_job_status_all' here
      * and `le_readline', or the "notifyle" option won't work. More precisely,
      * `handle_sigchld' must not be called from any other function until it is
      * called from `wait_for_input' during the line-editing. */
 
+    wchar_t *line;
+    inputresult_T result;
+
 #if YASH_ENABLE_LINEEDIT
     /* read a line using line editing */
     if (info->fileinfo->fd == STDIN_FILENO
-	    && shopt_lineedit != SHOPT_NOLINEEDIT) {
-	wchar_t *line;
-	inputresult_T result;
-
-	result = le_readline(prompt, true, &line);
-	if (result != INPUT_ERROR) {
-	    free_prompt(prompt);
-	    if (result == INPUT_OK) {
-		if (info->prompttype == 1)
-		    info->prompttype = 2;
-#if YASH_ENABLE_HISTORY
-		add_history(line);
-#endif
-		wb_catfree(buf, line);
-	    }
-	    return result;
-	}
+            && shopt_lineedit != SHOPT_NOLINEEDIT) {
+        result = le_readline(prompt, true, &line);
+        switch (result) {
+            case INPUT_OK:
+                goto success;
+            case INPUT_EOF:
+            case INPUT_INTERRUPTED:
+                goto done;
+            case INPUT_ERROR:
+                break; // Line-editing unavailable. Fall back to plain input.
+        }
     }
 #endif /* YASH_ENABLE_LINEEDIT */
 
     /* read a line without line editing */
     print_prompt(prompt.main);
     print_prompt(prompt.styler);
-    if (info->prompttype == 1)
-	info->prompttype = 2;
 
-    int result;
-#if YASH_ENABLE_HISTORY
-    size_t oldlen = buf->length;
-#endif
-    result = input_file(buf, info->fileinfo);
+    xwcsbuf_T linebuf;
+    wb_init(&linebuf);
+    result = input_file(&linebuf, info->fileinfo);
+    line = wb_towcs(&linebuf);
 
     print_prompt(PROMPT_RESET);
-    free_prompt(prompt);
 
-#if YASH_ENABLE_HISTORY
-    add_history(buf->contents + oldlen);
+#if YASH_ENABLE_LINEEDIT
+success:
 #endif
+    if (info->prompttype == 1)
+        info->prompttype = 2;
+    if (!posixly_correct)
+        line = post_prompt_command(line);
+#if YASH_ENABLE_HISTORY
+    add_history(line);
+#endif
+    wb_catfree(buf, line);
+#if YASH_ENABLE_LINEEDIT
+done:
+#endif
+    free_prompt(prompt);
     return result;
 }
 
@@ -306,27 +322,27 @@ struct promptset_T get_prompt(int type)
 
     wchar_t num;
     switch (type) {
-	case 1:   num = L'1';  break;
-	case 2:   num = L'2';  break;
-	case 4:   num = L'4';  break;
-	default:  assert(false);
+        case 1:   num = L'1';  break;
+        case 2:   num = L'2';  break;
+        case 4:   num = L'4';  break;
+        default:  assert(false);
     }
 
     wchar_t *prompt = expand_prompt_variable(num, L'\0');
     if (posixly_correct) {
-	if (type == 1)
-	    result.main = expand_ps1_posix(prompt);
-	else
-	    result.main = escapefree(prompt, L"\\");
+        if (type == 1)
+            result.main = expand_ps1_posix(prompt);
+        else
+            result.main = escapefree(prompt, L"\\");
 
-	result.right  = xwcsdup(L"");
-	result.styler = xwcsdup(L"");
-	result.predict = xwcsdup(L"");
+        result.right  = xwcsdup(L"");
+        result.styler = xwcsdup(L"");
+        result.predict = xwcsdup(L"");
     } else {
-	result.main = prompt;
-	result.right = expand_prompt_variable(num, L'R');
-	result.styler = expand_prompt_variable(num, L'S');
-	result.predict = expand_prompt_variable(num, L'P');
+        result.main = prompt;
+        result.right = expand_prompt_variable(num, L'R');
+        result.styler = expand_prompt_variable(num, L'S');
+        result.predict = expand_prompt_variable(num, L'P');
     }
 
     return result;
@@ -352,14 +368,14 @@ const wchar_t *get_prompt_variable(wchar_t num, wchar_t suffix)
 
     const wchar_t *value;
     if (!posixly_correct) {
-	value = getvar(name);
-	if (value != NULL)
-	    return value;
+        value = getvar(name);
+        if (value != NULL)
+            return value;
     }
 
     value = getvar(&name[5]);
     if (value != NULL)
-	return value;
+        return value;
 
     return L"";
 }
@@ -375,20 +391,20 @@ wchar_t *expand_ps1_posix(wchar_t *s)
 
     wb_init(&buf);
     for (size_t i = 0; s[i] != L'\0'; i++) {
-	if (s[i] == L'\\') {
-	    wb_wccat(&buf, L'\\');
-	    wb_wccat(&buf, L'\\');
-	} else if (s[i] == L'!') {
-	    if (s[i + 1] == L'!') {
-		wb_wccat(&buf, L'!');
-		i++;
-	    } else {
-		wb_wccat(&buf, L'\\');
-		wb_wccat(&buf, L'!');
-	    }
-	} else {
-	    wb_wccat(&buf, s[i]);
-	}
+        if (s[i] == L'\\') {
+            wb_wccat(&buf, L'\\');
+            wb_wccat(&buf, L'\\');
+        } else if (s[i] == L'!') {
+            if (s[i + 1] == L'!') {
+                wb_wccat(&buf, L'!');
+                i++;
+            } else {
+                wb_wccat(&buf, L'\\');
+                wb_wccat(&buf, L'!');
+            }
+        } else {
+            wb_wccat(&buf, s[i]);
+        }
     }
 
     free(s);
@@ -435,38 +451,38 @@ void print_prompt(const wchar_t *s)
 {
 #if YASH_ENABLE_LINEEDIT
     if (le_try_print_prompt(s))
-	return;
+        return;
 #endif
 
     xwcsbuf_T buf;
 
     wb_init(&buf);
     while (*s != L'\0') {
-	if (*s != L'\\') {
-	    wb_wccat(&buf, *s);
-	} else switch (*++s) {
-	    default:     wb_wccat(&buf, *s);       break;
-	    case L'\0':  wb_wccat(&buf, L'\\');    goto done;
-//	    case L'\\':  wb_wccat(&buf, L'\\');    break;
-	    case L'a':   wb_wccat(&buf, L'\a');    break;
-	    case L'e':   wb_wccat(&buf, L'\033');  break;
-	    case L'n':   wb_wccat(&buf, L'\n');    break;
-	    case L'r':   wb_wccat(&buf, L'\r');    break;
-	    case L'$':   wb_wccat(&buf, get_euid_marker());      break;
-	    case L'j':   wb_wprintf(&buf, L"%zu", job_count());  break;
+        if (*s != L'\\') {
+            wb_wccat(&buf, *s);
+        } else switch (*++s) {
+            default:     wb_wccat(&buf, *s);       break;
+            case L'\0':  wb_wccat(&buf, L'\\');    goto done;
+//          case L'\\':  wb_wccat(&buf, L'\\');    break;
+            case L'a':   wb_wccat(&buf, L'\a');    break;
+            case L'e':   wb_wccat(&buf, L'\033');  break;
+            case L'n':   wb_wccat(&buf, L'\n');    break;
+            case L'r':   wb_wccat(&buf, L'\r');    break;
+            case L'$':   wb_wccat(&buf, get_euid_marker());      break;
+            case L'j':   wb_wprintf(&buf, L"%zu", job_count());  break;
 #if YASH_ENABLE_HISTORY
-	    case L'!':   wb_wprintf(&buf, L"%u", next_history_number());  break;
+            case L'!':   wb_wprintf(&buf, L"%u", next_history_number());  break;
 #endif
-	    case L'[':
-	    case L']':
-		break;
-	    case L'f':
-		while (iswalnum(*++s));
-		if (*s == L'.')
-		    s++;
-		continue;
-	}
-	s++;
+            case L'[':
+            case L']':
+                break;
+            case L'f':
+                while (iswalnum(*++s));
+                if (*s == L'.')
+                    s++;
+                continue;
+        }
+        s++;
     }
 done:
     fprintf(stderr, "%ls", buf.contents);
@@ -479,20 +495,53 @@ wchar_t get_euid_marker(void)
     return geteuid() == 0 ? L'#' : L'$';
 }
 
+/* Executes $POST_PROMPT_COMMAND, if any.
+ * `line' is the just input command line, which will be assigned to $COMMAND
+ * during the execution. The post-prompt command may modify or unset the
+ * variable. The final value of the variable is returned as a newly malloced
+ * string. `line' is freed in this function. */
+wchar_t *post_prompt_command(wchar_t *line)
+{
+    // If `line` ends with a newline, trim it here and append it back later.
+    size_t linelen = wcslen(line);
+    bool newline = linelen > 0 && line[linelen - 1] == L'\n';
+    if (newline)
+        line[linelen - 1] = L'\0';
+
+    open_new_environment(false);
+    set_positional_parameters((void *[]) { NULL });
+    set_variable(L VAR_COMMAND, line, SCOPE_LOCAL, false);
+
+    exec_variable_as_auxiliary_(VAR_POST_PROMPT_COMMAND);
+    const wchar_t *c = getvar(L VAR_COMMAND);
+    if (c == NULL)
+        c = L"";
+
+    xwcsbuf_T linebuf;
+    wb_init(&linebuf);
+    wb_cat(&linebuf, c);
+    if (newline)
+        wb_wccat(&linebuf, L'\n');
+
+    close_current_environment();
+
+    return wb_towcs(&linebuf);
+}
+
 /* Unsets O_NONBLOCK flag of the specified file descriptor.
  * If `fd' is negative, does nothing.
  * Returns true if successful. On error, `errno' is set and false is returned.*/
 bool unset_nonblocking(int fd)
 {
     if (fd >= 0) {
-	int flags = fcntl(fd, F_GETFL);
-	if (flags < 0)
-	    return false;
-	if (flags & O_NONBLOCK)
-	    return fcntl(fd, F_SETFL, flags & ~O_NONBLOCK) != -1;
+        int flags = fcntl(fd, F_GETFL);
+        if (flags < 0)
+            return false;
+        if (flags & O_NONBLOCK)
+            return fcntl(fd, F_SETFL, flags & ~O_NONBLOCK) != -1;
     }
     return true;
 }
 
 
-/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
+/* vim: set ts=8 sts=4 sw=4 et tw=80: */
