@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* job.h: job control */
-/* (C) 2007-2017 magicant */
+/* (C) 2007-2025 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,7 +71,7 @@ extern void init_job(void);
 
 extern void set_active_job(job_T *job)
     __attribute__((nonnull));
-extern void add_job(_Bool current);
+extern size_t add_job(_Bool current);
 extern void remove_job(size_t jobnumber);
 extern void remove_job_nofitying_signal(size_t jobnumber);
 extern void remove_all_jobs(void);
@@ -83,7 +83,7 @@ extern size_t stopped_job_count(void)
 
 extern void do_wait(void);
 extern int wait_for_job(size_t jobnumber, _Bool return_on_stop,
-	_Bool interruptible, _Bool return_on_trap);
+        _Bool interruptible, _Bool return_on_trap);
 extern wchar_t **wait_for_child(pid_t cpid, pid_t cpgid, _Bool return_on_stop);
 extern pid_t get_job_pgid(const wchar_t *jobname)
     __attribute__((pure));
@@ -128,4 +128,4 @@ extern const char disown_help[], disown_syntax[];
 #endif /* YASH_JOB_H */
 
 
-/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
+/* vim: set ts=8 sts=4 sw=4 et tw=80: */
