@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* parser.h: syntax parser */
-/* (C) 2007-2018 magicant */
+/* (C) 2007-2025 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -76,48 +76,50 @@ typedef struct command_T {
     unsigned long     c_lineno;   /* line number */
     struct redir_T   *c_redirs;   /* redirections */
     union {
-	struct {
-	    struct assign_T *assigns;  /* assignments */
-	    void           **words;    /* command name and arguments */
-	} simplecommand;
-	struct and_or_T     *subcmds;  /* contents of command group */
-	struct ifcommand_T  *ifcmds;   /* contents of if command */
-	struct {
-	    wchar_t         *forname;  /* loop variable of for loop */
-	    void           **forwords; /* words assigned to loop variable */
-	    struct and_or_T *forcmds;  /* commands executed in for loop */
-	} forloop;
-	struct {
-	    _Bool            whltype;  /* 1 for while loop, 0 for until */
-	    struct and_or_T *whlcond;  /* loop condition of while/until loop */
-	    struct and_or_T *whlcmds;  /* commands executed in loop */
-	} whileloop;
-	struct {
-	    struct wordunit_T *casword;   /* word compared to case patterns */
-	    struct caseitem_T *casitems;  /* pairs of patterns and commands */
-	} casecommand;
-	struct dbexp_T      *dbexp;    /* double-bracket command expression */
-	struct {
-	    struct wordunit_T *funcname;  /* name of function */
-	    struct command_T  *funcbody;  /* body of function */
-	} funcdef;
+        struct {
+            struct assign_T *assigns;     /* assignments */
+            void           **words;       /* command name and arguments */
+            _Bool            isdeclutil;  /* is a declaration utility? */
+        } simplecommand;
+        struct and_or_T     *subcmds;  /* contents of command group */
+        struct ifcommand_T  *ifcmds;   /* contents of if command */
+        struct {
+            wchar_t         *forname;  /* loop variable of for loop */
+            void           **forwords; /* words assigned to loop variable */
+            struct and_or_T *forcmds;  /* commands executed in for loop */
+        } forloop;
+        struct {
+            _Bool            whltype;  /* 1 for while loop, 0 for until */
+            struct and_or_T *whlcond;  /* loop condition of while/until loop */
+            struct and_or_T *whlcmds;  /* commands executed in loop */
+        } whileloop;
+        struct {
+            struct wordunit_T *casword;   /* word compared to case patterns */
+            struct caseitem_T *casitems;  /* pairs of patterns and commands */
+        } casecommand;
+        struct dbexp_T      *dbexp;    /* double-bracket command expression */
+        struct {
+            struct wordunit_T *funcname;  /* name of function */
+            struct command_T  *funcbody;  /* body of function */
+        } funcdef;
     } c_content;
 } command_T;
-#define c_assigns  c_content.simplecommand.assigns
-#define c_words    c_content.simplecommand.words
-#define c_subcmds  c_content.subcmds
-#define c_ifcmds   c_content.ifcmds
-#define c_forname  c_content.forloop.forname
-#define c_forwords c_content.forloop.forwords
-#define c_forcmds  c_content.forloop.forcmds
-#define c_whltype  c_content.whileloop.whltype
-#define c_whlcond  c_content.whileloop.whlcond
-#define c_whlcmds  c_content.whileloop.whlcmds
-#define c_casword  c_content.casecommand.casword
-#define c_casitems c_content.casecommand.casitems
-#define c_dbexp    c_content.dbexp
-#define c_funcname c_content.funcdef.funcname
-#define c_funcbody c_content.funcdef.funcbody
+#define c_assigns    c_content.simplecommand.assigns
+#define c_words      c_content.simplecommand.words
+#define c_isdeclutil c_content.simplecommand.isdeclutil
+#define c_subcmds    c_content.subcmds
+#define c_ifcmds     c_content.ifcmds
+#define c_forname    c_content.forloop.forname
+#define c_forwords   c_content.forloop.forwords
+#define c_forcmds    c_content.forloop.forcmds
+#define c_whltype    c_content.whileloop.whltype
+#define c_whlcond    c_content.whileloop.whlcond
+#define c_whlcmds    c_content.whileloop.whlcmds
+#define c_casword    c_content.casecommand.casword
+#define c_casitems   c_content.casecommand.casitems
+#define c_dbexp      c_content.dbexp
+#define c_funcname   c_content.funcdef.funcname
+#define c_funcbody   c_content.funcdef.funcbody
 /* `c_words' and `c_forwords' are NULL-terminated arrays of pointers to
  * `wordunit_T' that are cast to `void *'.
  * If `c_forwords' is NULL, the for loop doesn't have the "in" clause.
@@ -131,11 +133,19 @@ typedef struct ifcommand_T {
 } ifcommand_T;
 /* For an "else" clause, `next' and `ic_condition' are NULL. */
 
+/* type of an case item terminator symbol */
+typedef enum {
+    CC_BREAK,    // ;;
+    CC_FALLTHRU, // ;&
+    CC_CONTINUE, // ;| aka ;;&
+} casecont_T;
+
 /* patterns and commands of a case command */
 typedef struct caseitem_T {
     struct caseitem_T *next;
     void             **ci_patterns;  /* patterns to do matching */
     struct and_or_T   *ci_commands;  /* commands executed if match succeeds */
+    casecont_T         ci_cont;      /* terminator symbol type */
 } caseitem_T;
 /* `ci_patterns' is a NULL-terminated array of pointers to `wordunit_T' that are
  * cast to `void *'. */
@@ -169,8 +179,8 @@ typedef struct dbexp_T {
 typedef struct embedcmd_T {
     _Bool is_preparsed;
     union {
-	wchar_t         *unparsed;
-	struct and_or_T *preparsed;
+        wchar_t         *unparsed;
+        struct and_or_T *preparsed;
     } value;
 } embedcmd_T;
 
@@ -187,10 +197,10 @@ typedef struct wordunit_T {
     struct wordunit_T *next;
     wordunittype_T     wu_type;
     union {
-	wchar_t           *string;  /* string (including quotes) */
-	struct paramexp_T *param;   /* parameter expansion */
-	struct embedcmd_T  cmdsub;  /* command substitution */
-	struct wordunit_T *arith;   /* expression for arithmetic expansion */
+        wchar_t           *string;  /* string (including quotes) */
+        struct paramexp_T *param;   /* parameter expansion */
+        struct embedcmd_T  cmdsub;  /* command substitution */
+        struct wordunit_T *arith;   /* expression for arithmetic expansion */
     } wu_value;
 } wordunit_T;
 #define wu_string wu_value.string
@@ -243,8 +253,8 @@ typedef enum {
 typedef struct paramexp_T {
     paramexptype_T pe_type;
     union {
-	wchar_t           *name;
-	struct wordunit_T *nest;
+        wchar_t           *name;
+        struct wordunit_T *nest;
     } pe_value;
     struct wordunit_T *pe_start, *pe_end;
     struct wordunit_T *pe_match, *pe_subst;
@@ -271,8 +281,8 @@ typedef struct assign_T {
     assigntype_T a_type;
     wchar_t *a_name;
     union {
-	struct wordunit_T *scalar;
-	void **array;          
+        struct wordunit_T *scalar;
+        void **array;          
     } a_value;  /* value to assign */
 } assign_T;
 #define a_scalar a_value.scalar
@@ -303,12 +313,12 @@ typedef struct redir_T {
     redirtype_T rd_type;
     int rd_fd;  /* file descriptor to redirect */
     union {
-	struct wordunit_T *filename;
-	struct {
-	    wchar_t *hereend;  /* token indicating end of here-document */
-	    struct wordunit_T *herecontent;  /* contents of here-document */
-	} heredoc;
-	struct embedcmd_T command;
+        struct wordunit_T *filename;
+        struct {
+            wchar_t *hereend;  /* token indicating end of here-document */
+            struct wordunit_T *herecontent;  /* contents of here-document */
+        } heredoc;
+        struct embedcmd_T command;
     } rd_value;
 } redir_T;
 #define rd_filename    rd_value.filename
@@ -336,6 +346,7 @@ typedef struct parseparam_T {
     inputfunc_T *input;   /* input function */
     void *inputinfo;      /* pointer passed to the input function */
     _Bool interactive;    /* input is interactive? */
+    _Bool is_dot_builtin; /* input is for the dot built-in? */
     inputresult_T lastinputresult;  /* last return value of input function */
 } parseparam_T;
 /* If `interactive' is true, `input' is `input_interactive' and `inputinfo' is a
@@ -348,7 +359,7 @@ typedef enum parseresult_T {
 
 
 extern parseresult_T read_and_parse(
-	parseparam_T *info, and_or_T **restrict resultp)
+        parseparam_T *info, and_or_T **restrict resultp)
     __attribute__((nonnull,warn_unused_result));
 
 extern _Bool parse_string(parseparam_T *info, wordunit_T **restrict resultp)
@@ -362,7 +373,9 @@ extern _Bool is_portable_name_char(wchar_t c)
 extern _Bool is_name_char(wchar_t c)
     __attribute__((pure));
 extern _Bool is_name(const wchar_t *s)
-    __attribute__((pure));
+    __attribute__((nonnull,pure));
+extern _Bool is_assignment_prefix(const wchar_t *s)
+    __attribute__((nonnull,pure));
 extern _Bool is_keyword(const wchar_t *s)
     __attribute__((nonnull,pure));
 extern _Bool is_token_delimiter_char(wchar_t c)
@@ -397,4 +410,4 @@ command_T *comsdup(command_T *c)
 #endif /* YASH_PARSER_H */
 
 
-/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
+/* vim: set ts=8 sts=4 sw=4 et tw=80: */
