@@ -1,5 +1,5 @@
 # run-test.sh: runs a set of test cases
-# (C) 2016-2019 magicant
+# (C) 2016-2025 magicant
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,9 +38,9 @@ eprintf() {
 absolute()
 case "$1" in
     (/*)
-	printf '%s\n' "$1";;
+        printf '%s\n' "$1";;
     (*)
-	printf '%s/%s' "${PWD%/}" "$1";;
+        printf '%s/%s' "${PWD%/}" "$1";;
 esac
 
 ##### Script startup
@@ -57,12 +57,12 @@ remove_work_dir="true"
 use_valgrind="false"
 while getopts rv opt; do
     case $opt in
-	(r)
-	    remove_work_dir="false";;
-	(v)
-	    use_valgrind="true";;
-	(*)
-	    exit 64 # sysexits.h EX_USAGE
+        (r)
+            remove_work_dir="false";;
+        (v)
+            use_valgrind="true";;
+        (*)
+            exit 64 # sysexits.h EX_USAGE
     esac
 done
 shift "$((OPTIND-1))"
@@ -77,12 +77,12 @@ exec >|"${test_file%.*}.trs"
 export LC_CTYPE="${LC_ALL-${LC_CTYPE-$LANG}}"
 export LANG=C
 export YASH_LOADPATH= # ignore default yashrc
-unset -v CDPATH COLUMNS COMMAND_NOT_FOUND_HANDLER DIRSTACK ECHO_STYLE ENV
-unset -v FCEDIT HANDLED HISTFILE HISTRMDUP HISTSIZE HOME IFS LC_ALL
+unset -v CDPATH COLUMNS COMMAND COMMAND_NOT_FOUND_HANDLER DIRSTACK ECHO_STYLE
+unset -v ENV FCEDIT HANDLED HISTFILE HISTRMDUP HISTSIZE HOME IFS LC_ALL
 unset -v LC_COLLATE LC_MESSAGES LC_MONETARY LC_NUMERIC LC_TIME LINES MAIL
-unset -v MAILCHECK MAILPATH NLSPATH OLDPWD PROMPT_COMMAND
+unset -v MAILCHECK MAILPATH NLSPATH OLDPWD POST_PROMPT_COMMAND PROMPT_COMMAND
 unset -v PS1 PS1R PS1S PS2 PS2R PS2S PS3 PS3R PS3S PS4 PS4R PS4S 
-unset -v RANDOM TERM YASH_AFTER_CD YASH_LE_TIMEOUT YASH_VERSION
+unset -v RANDOM TERM XDG_CONFIG_HOME YASH_AFTER_CD YASH_LE_TIMEOUT YASH_VERSION
 unset -v A B C D E F G H I J K L M N O P Q R S T U V W X Y Z _
 unset -v a b c d e f g h i j k l m n o p q r s t u v w x y z
 unset -v posix skip
@@ -108,11 +108,11 @@ mkdir "$work_dir"
 
 {
     if diff -U 10000 /dev/null /dev/null; then
-	diff_opt='-U 10000'
+        diff_opt='-U 10000'
     elif diff -C 10000 /dev/null /dev/null; then
-	diff_opt='-C 10000'
+        diff_opt='-C 10000'
     else
-	diff_opt=''
+        diff_opt=''
     fi
 } >/dev/null 2>&1
 
@@ -124,12 +124,12 @@ setup_script=""
 # Otherwise, the first argument is added as the script.
 setup() {
     case "${1--}" in
-	(-)
-	    setup "$(cat)"
-	    ;;
-	(-d)
-	    setup <<\END
-_empty= _sp=' ' _tab='	' _nl='
+        (-)
+            setup "$(cat)"
+            ;;
+        (-d)
+            setup <<\END
+_empty= _sp=' ' _tab='  ' _nl='
 '
 echoraw() {
     printf '%s\n' "$*"
@@ -139,14 +139,28 @@ bracket() {
     echo
 }
 END
-	    ;;
-	(*)
-	    setup_script="$setup_script
+            ;;
+        (*)
+            setup_script="$setup_script
 $1"
-	    ;;
+            ;;
     esac
 }
 
+macos_kill_workaround()
+if [ "$(uname)" = Darwin ]; then
+    # On macOS, kill(2) does not appear to run any signal handlers
+    # synchronously, making it impossible for the shell to respond to self-sent
+    # signals at a predictable time. To work around this issue, we call the kill
+    # built-in in a nested subshell on macOS. The subshell and the dummy sleep
+    # command generate some more SIGCHLD signals that must be handled by the
+    # shell, which makes it more likely that the signal sent by the kill built-in
+    # is delivered to the shell before the subshell exits.
+    setup <<'__EOF__'
+kill() (trap 'sleep 0' EXIT; (trap 'sleep 0' EXIT; (trap 'sleep 0' EXIT; command kill "$@")))
+__EOF__
+fi
+
 # Invokes the testee.
 # If the "posix" variable is defined non-empty, the testee is invoked as "sh".
 # If the "use_valgrind" variable is true, Valgrind is used to run the testee,
@@ -156,18 +170,18 @@ testee() (
 )
 exec_testee() {
     if [ "${posix:+set}" = set ]; then
-	testee="$testee_sh"
-	export TESTEE="$testee"
+        testee="$testee_sh"
+        export TESTEE="$testee"
     fi
     if ! "$use_valgrind"; then
-	exec "$testee" "$@"
+        exec "$testee" "$@"
     else
-	test -r "$abs_suppressions" || abs_suppressions=
-	exec valgrind --leak-check=full --vgdb=no --log-fd=17 \
-	    ${abs_suppressions:+--suppressions="$abs_suppressions"} \
-	    --gen-suppressions=all \
-	    "$testee" "$@" \
-	    17>>"${valgrind_file-0.valgrind}"
+        test -r "$abs_suppressions" || abs_suppressions=
+        exec valgrind --leak-check=full --vgdb=no --log-fd=17 \
+            ${abs_suppressions:+--suppressions="$abs_suppressions"} \
+            --gen-suppressions=all \
+            "$testee" "$@" \
+            17>>"${valgrind_file-0.valgrind}"
     fi
 }
 
@@ -191,6 +205,9 @@ exec_testee() {
 # is any non-zero exit status. If <expected_exit_status> is a signal name (w/o
 # the SIG-prefix), the testee is expected to be killed by the signal.
 #
+# If the "-f" option is specified, the test result is inverted. This is useful
+# for testing unimplemented features and unfixed bugs.
+#
 # The first operand is used as the name of the test case.
 # The remaining operands are passed as arguments to the testee.
 #
@@ -201,23 +218,26 @@ testcase() {
     OPTIND=1
     diagnostic_required="false"
     expected_exit_status=""
-    while getopts de: opt; do
-	case $opt in
-	    (d)
-		diagnostic_required="true";;
-	    (e)
-		expected_exit_status="$OPTARG";;
-	    (*)
-		return 64 # sysexits.h EX_USAGE
-	esac
+    should_succeed="true"
+    while getopts de:f opt; do
+        case $opt in
+            (d)
+                diagnostic_required="true";;
+            (e)
+                expected_exit_status="$OPTARG";;
+            (f)
+                should_succeed="false";;
+            (*)
+                return 64 # sysexits.h EX_USAGE
+        esac
     done
     shift "$((OPTIND-1))"
     test_case_name="${1:?unnamed test case \($test_file:$test_lineno\)}"
     shift 1
 
     log_stdout() {
-	printf '%%%%%% %s: %s:%d: %s\n' \
-	    "$1" "$test_file" "$test_lineno" "$test_case_name"
+        printf '%%%%%% %s: %s:%d: %s\n' \
+            "$1" "$test_file" "$test_lineno" "$test_case_name"
     }
 
     in_file="$test_lineno.in"
@@ -227,26 +247,26 @@ testcase() {
 
     # prepare input file
     {
-	if [ "$setup_script" ]; then
-	    printf '%s\n' "$setup_script"
-	fi
-	cat <&3
+        if [ "$setup_script" ]; then
+            printf '%s\n' "$setup_script"
+        fi
+        cat <&3
     } >"$in_file"
     chmod u+r "$in_file"
 
     if [ "${skip-}" ]; then
-	log_stdout SKIPPED
-	echo
-	return
+        log_stdout SKIPPED
+        echo
+        return
     fi
 
     if [ -e "$out_file" ]; then
-	printf 'Output file %s already exists.\n' "$out_file"
-	return 1
+        printf 'Output file %s already exists.\n' "$out_file"
+        return 1
     fi
     if [ -e "$err_file" ]; then
-	printf 'Output file %s already exists.\n' "$err_file"
-	return 1
+        printf 'Output file %s already exists.\n' "$err_file"
+        return 1
     fi
 
     # run the testee
@@ -267,98 +287,116 @@ testcase() {
 
     # check exit status
     exit_status_fail() {
-	failed="true"
-	eprintf '%s:%d: %s: exit status mismatch\n' \
-	    "$test_file" "$test_lineno" "$test_case_name"
+        failed="true"
+        if "$should_succeed"; then
+            eprintf '%s:%d: %s: exit status mismatch\n' \
+                "$test_file" "$test_lineno" "$test_case_name"
+        fi
     }
     case "$expected_exit_status" in
-	('')
-	    ;;
-	(n)
-	    printf '%% exit status: expected=non-zero actual=%d\n\n' \
-		"$actual_exit_status"
-	    if [ "$actual_exit_status" -eq 0 ]; then
-		exit_status_fail
-	    fi
-	    ;;
-	([[:alpha:]]*)
-	    printf '%% exit status: expected=%s ' "$expected_exit_status"
-	    if [ "$actual_exit_status" -le 128 ] ||
-		    ! actual_signal="$(kill -l "$actual_exit_status" \
-			2>/dev/null)"; then
-		printf 'actual=%d\n\n' "$actual_exit_status"
-		exit_status_fail
-	    else
-		printf 'actual=%d(%s)\n\n' \
-		    "$actual_exit_status" "$actual_signal"
-		if [ "$actual_signal" != "$expected_exit_status" ]; then
-		    exit_status_fail
-		fi
-	    fi
-	    ;;
-	(*)
-	    printf '%% exit status: expected=%d actual=%d\n\n' \
-		"$expected_exit_status" "$actual_exit_status"
-	    if [ "$actual_exit_status" -ne "$expected_exit_status" ]; then
-		exit_status_fail
-	    fi
-	    ;;
+        ('')
+            ;;
+        (n)
+            printf '%% exit status: expected=non-zero actual=%d\n\n' \
+                "$actual_exit_status"
+            if [ "$actual_exit_status" -eq 0 ]; then
+                exit_status_fail
+            fi
+            ;;
+        ([[:alpha:]]*)
+            printf '%% exit status: expected=%s ' "$expected_exit_status"
+            if [ "$actual_exit_status" -le 128 ] ||
+                    ! actual_signal="$(kill -l "$actual_exit_status" \
+                        2>/dev/null)"; then
+                printf 'actual=%d\n\n' "$actual_exit_status"
+                exit_status_fail
+            else
+                printf 'actual=%d(%s)\n\n' \
+                    "$actual_exit_status" "$actual_signal"
+                if [ "$actual_signal" != "$expected_exit_status" ]; then
+                    exit_status_fail
+                fi
+            fi
+            ;;
+        (*)
+            printf '%% exit status: expected=%d actual=%d\n\n' \
+                "$expected_exit_status" "$actual_exit_status"
+            if [ "$actual_exit_status" -ne "$expected_exit_status" ]; then
+                exit_status_fail
+            fi
+            ;;
     esac
 
     # check standard output
     if { <&4; } 2>/dev/null; then
-	printf '%% standard output diff:\n'
-	if ! diff $diff_opt - "$out_file" <&4; then
-	    failed="true"
-	    eprintf '%s:%d: %s: standard output mismatch\n' \
-		"$test_file" "$test_lineno" "$test_case_name"
-	fi
-	echo
+        printf '%% standard output diff:\n'
+        if ! diff $diff_opt - "$out_file" <&4; then
+            failed="true"
+            if "$should_succeed"; then
+                eprintf '%s:%d: %s: standard output mismatch\n' \
+                    "$test_file" "$test_lineno" "$test_case_name"
+            fi
+        fi
+        echo
     fi
 
     # check standard error
     if "$diagnostic_required"; then
-	printf '%% standard error (expecting non-empty output):\n'
-	cat "$err_file"
-	if ! [ -s "$err_file" ]; then
-	    failed="true"
-	    eprintf '%s:%d: %s: standard error mismatch\n' \
-		"$test_file" "$test_lineno" "$test_case_name"
-	fi
-	echo
+        printf '%% standard error (expecting non-empty output):\n'
+        cat "$err_file"
+        if ! [ -s "$err_file" ]; then
+            failed="true"
+            if "$should_succeed"; then
+                eprintf '%s:%d: %s: standard error mismatch\n' \
+                    "$test_file" "$test_lineno" "$test_case_name"
+            fi
+        fi
+        echo
     elif { <&5; } 2>/dev/null; then
-	printf '%% standard error diff:\n'
-	if ! diff $diff_opt - "$err_file" <&5; then
-	    failed="true"
-	    eprintf '%s:%d: %s: standard error mismatch\n' \
-		"$test_file" "$test_lineno" "$test_case_name"
-	fi
-	echo
+        printf '%% standard error diff:\n'
+        if ! diff $diff_opt - "$err_file" <&5; then
+            failed="true"
+            if "$should_succeed"; then
+                eprintf '%s:%d: %s: standard error mismatch\n' \
+                    "$test_file" "$test_lineno" "$test_case_name"
+            fi
+        fi
+        echo
     fi
 
     # check Valgrind results
     if [ -f "$valgrind_file" ]; then
-	chmod a+r "$valgrind_file"
-	printf '%% Valgrind log:\n'
-	cat "$valgrind_file"
-	echo
-	if grep -q 'valgrind: fatal error:' "$valgrind_file"; then
-	    # There was an error in Valgrind. Treat this test case as skipped.
-	    log_stdout SKIPPED
-	    echo
-	    return
-	fi
-	if grep 'ERROR SUMMARY:' "$valgrind_file" | grep -qv ' 0 errors'; then
-	    failed="true"
-	    eprintf '%s:%d: %s: Valgrind detected error\n' \
-		"$test_file" "$test_lineno" "$test_case_name"
-	fi
+        chmod a+r "$valgrind_file"
+        printf '%% Valgrind log:\n'
+        cat "$valgrind_file"
+        echo
+        if grep -q 'valgrind: fatal error:' "$valgrind_file"; then
+            # There was an error in Valgrind. Treat this test case as skipped.
+            log_stdout SKIPPED
+            echo
+            return
+        fi
+        if grep 'ERROR SUMMARY:' "$valgrind_file" | grep -qv ' 0 errors'; then
+            failed="true"
+            eprintf '%s:%d: %s: Valgrind detected error\n' \
+                "$test_file" "$test_lineno" "$test_case_name"
+        fi
     fi
 
-    if "$failed"; then
-	log_stdout FAILED
+    if "$should_succeed"; then
+        if "$failed"; then
+            log_stdout 'ERROR[FAILED]'
+        else
+            log_stdout 'OK[PASSED]'
+        fi
     else
-	log_stdout PASSED
+        if "$failed"; then
+            log_stdout 'OK[FAILED_AS_EXPECTED]'
+        else
+            log_stdout 'ERROR[PASSED_UNEXPECTEDLY]'
+            eprintf '%s:%d: %s: passed unexpectedly\n' \
+                "$test_file" "$test_lineno" "$test_case_name"
+        fi
     fi
     echo
 }
@@ -389,4 +427,4 @@ export TESTEE="$testee"
 . "$abs_test_file"
 )
 
-# vim: set ts=8 sts=4 sw=4 noet:
+# vim: set ts=8 sts=4 sw=4 et:
