@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* variable.h: deals with shell variables and parameters */
-/* (C) 2007-2020 magicant */
+/* (C) 2007-2025 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@ extern char **environ;
 /* variable names */
 #define VAR_CDPATH                    "CDPATH"
 #define VAR_COLUMNS                   "COLUMNS"
+#define VAR_COMMAND                   "COMMAND"
 #define VAR_COMMAND_NOT_FOUND_HANDLER "COMMAND_NOT_FOUND_HANDLER"
 #define VAR_DIRSTACK                  "DIRSTACK"
 #define VAR_ECHO_STYLE                "ECHO_STYLE"
@@ -57,6 +58,7 @@ extern char **environ;
 #define VAR_OPTARG                    "OPTARG"
 #define VAR_OPTIND                    "OPTIND"
 #define VAR_PATH                      "PATH"
+#define VAR_POST_PROMPT_COMMAND       "POST_PROMPT_COMMAND"
 #define VAR_PPID                      "PPID"
 #define VAR_PROMPT_COMMAND            "PROMPT_COMMAND"
 #define VAR_PS1                       "PS1"
@@ -67,6 +69,7 @@ extern char **environ;
 #define VAR_TARGETWORD                "TARGETWORD"
 #define VAR_TERM                      "TERM"
 #define VAR_WORDS                     "WORDS"
+#define VAR_XDG_CONFIG_HOME           "XDG_CONFIG_HOME"
 #define VAR_YASH_AFTER_CD             "YASH_AFTER_CD"
 #define VAR_YASH_LE_TIMEOUT           "YASH_LE_TIMEOUT"
 #define VAR_YASH_LOADPATH             "YASH_LOADPATH"
@@ -92,19 +95,19 @@ typedef enum scope_T {
     SCOPE_GLOBAL, SCOPE_LOCAL, SCOPE_TEMP,
 } scope_T;
 extern _Bool set_variable(
-	const wchar_t *name, wchar_t *value, scope_T scope, _Bool export)
+        const wchar_t *name, wchar_t *value, scope_T scope, _Bool export)
     __attribute__((nonnull(1)));
 extern struct variable_T *set_array(
-	const wchar_t *name, size_t count, void **values,
-	scope_T scope, _Bool export)
+        const wchar_t *name, size_t count, void **values,
+        scope_T scope, _Bool export)
     __attribute__((nonnull));
 extern _Bool set_array_element(
-	const wchar_t *name, size_t index, wchar_t *value)
+        const wchar_t *name, size_t index, wchar_t *value)
     __attribute__((nonnull));
 extern void set_positional_parameters(void *const *values)
     __attribute__((nonnull));
 extern _Bool do_assignments(
-	const struct assign_T *assigns, _Bool temp, _Bool export);
+        const struct assign_T *assigns, _Bool temp, _Bool export);
 
 struct get_variable_T {
     enum { GV_NOTFOUND, GV_SCALAR, GV_ARRAY, GV_ARRAY_CONCAT, } type;
@@ -135,8 +138,8 @@ extern struct command_T *get_function(const wchar_t *name)
 
 #if YASH_ENABLE_DIRSTACK
 extern _Bool parse_dirstack_index(
-	const wchar_t *restrict indexstr, size_t *restrict indexp,
-	const wchar_t **restrict entryp, _Bool printerror)
+        const wchar_t *restrict indexstr, size_t *restrict indexp,
+        const wchar_t **restrict entryp, _Bool printerror)
     __attribute__((nonnull));
 #endif
 
@@ -192,10 +195,10 @@ extern const char pushd_help[], pushd_syntax[];
 extern const struct xgetopt_T pushd_options[];
 #if YASH_ENABLE_DIRSTACK
 # define cd_options  (&pushd_options[1])
-# define pwd_options (&pushd_options[2])
+# define pwd_options (&pushd_options[3])
 #else
 # define cd_options  pushd_options
-# define pwd_options (&pushd_options[1])
+# define pwd_options (&pushd_options[2])
 #endif
 
 extern int popd_builtin(int argc, void **argv)
@@ -215,4 +218,4 @@ extern const struct xgetopt_T dirs_options[];
 #endif /* YASH_VARIABLE_H */
 
 
-/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
+/* vim: set ts=8 sts=4 sw=4 et tw=80: */
